"""
PYTHON-SKRIPT ZUM PARAMETRIEREN DES IC Si5351A 
----------------------------------------------

AUSGANGSFREQUENZ: 10 MHz
------------------------

Folgender Python-Code laedt Werte in die Register des Si5351A,
so dass der IC an seinem CLK0-Ausgang ein 10MHz-Signal erzeugt.
Wegen der Ungenauigkeit des auf dem Si5351A-Breakout Boards befindlichen
Quarzes (ca. 60ppm)werden nicht exakt 10MHz erzeugt.
Erfahrungsgemaess liegt die Frequenz bei ca. 10.000.600 Hz

Um anstatt 10MHz eine andere Frequenz zu erzeugen, sind die Inhalte
der frequenzbestimmenden Register (s.u.) zu aendern.

"""


import time                         #
from machine import Pin, I2C        #Bibliothek speziell fuer die Hardware des Mikrocontrollers
from array import *                 #Bibliothek erforderlich, da Register und deren Inhalte in einer Liste hinterlegt sind


###############################################################################
# Konstanten

# Feste I2C-Adresse des Si5351A, siehe Datenblatt des IC-Herstellers
Si5351A_ADDR = 0x60



###############################################################################
# PARAMETRIERUNG DER Si5351A-REGISTER:
# ------------------------------------

regs = [                          #ab hier beginnt die Liste aller Si5351A-Register inkl. deren Inhalte

#Zu Beginn alle CLK-Ausgaenge (CLK0, CLK1, CLK2) deaktvieren
#-----------------------------------------------------------
#Register 3, Register-Inhalt 0x07:  "Output Enable Control": Ausgaenge CLK0/CLK1/CLK2 deaktivieren
[3, 0x07],      


#Ausgangstreiber der CLK-Ausgaenge deaktivieren und Signalfluss konfigurieren (XTAL -> PLL A -> Multisynth 0 -> CLK0) 
#Blockschaltbild zum Signalfluss siehe Datenlatt des IC-Herstellers
#--------------------------------------------------------------------------------------------------------------------

#Register 15, Register-Inhalt 0x00:	"PLL Input Source" - fuer PLL A und B den Quarz (XTAL) als Eingangsquelle gewaehlt, Eingangsteiler = 1
[15, 0x00],


#       Register 16, Register-Inhalt 0x8C:  "CLK0 Control"
#       Der CLK0 AusgangsTREIBER verbleibt aktiviert (dient im Regelfall zur Reduzierung des Energieverbrauchs)
#       MS0 wird in "fractional division mode" betrieben (somit kein Integer Mode)
#       PLL A ist Quelle fuer MultiSynth 0
#       Signal am Ausgang CLK0 wird nicht invertiert     
#       Der MultiSynth 0 ist die Signal-Quelle fuer den CLK0-Ausgang
#       Strom am Ausgangstreiber CLK0: 2mA
[16, 0x0C],
 
 
 
 
 

#Einstellung der Grundfunktionen des Si5351A
#-------------------------------------------

#Register 9, Register-Inhalt 0xFF:  "Output Enable Bit Control" - der HW-Pin "OEB" existiert NICHT am Si5351A im Gehause 10-MSOP, somit ignorieren
[9, 0xFF], 

#Register 24, Register-Inhalt 0x00: "Clock x Disable State": Ausgaenge CLK0, CLK1, CLK2 immer im deaktivierten Zustand auf LOW-Pegel setzen
[24, 0x00],
  
#Register 183, Register-Inhalt 0xD2: "Crystal Load Capacitance Selection": Quarz-Kapazität: 10pF (default-Wert) willkuerlich gewaehlt,
#da keine Angabe fuer Breakout Board vorliegt (hierzu den Text im Datenblatt beachten: "Bits 5:0 should be written to 010010b")
[183, 0xD2],  
  


#Die Frequenzbestimmenden Register des Si5351A laden (beispielhaft fuer Frequenz 10MHz an Ausgang CLK0)
#------------------------------------------------------------------------------------------------------

#Parametrierung des PLL A Teilers
[26, 0x00],
[27, 0x01],
[28, 0x00],
[29, 0x0B],
[30, 0x00],
[31, 0x00],
[32, 0x00],
[33, 0x00],
  
#Parametrierung des Multisynth Ausgangsteilers 
[42, 0x00],
[43, 0x01],
[44, 0x00],
[45, 0x1E],
[46, 0x80],
[47, 0x00],
[48, 0x00],
[49, 0x00],  
  

#Hier ist das Ende der frequenzbestimmenden Register


#Phase Lock Loop PLL A mittels Soft-Reset zuruecksetzen (Reset ist lt. Datenblatt erforderlich)
#----------------------------------------------------------------------------------------------
[177, 0x20],
#Register 177, Register-Inhalt 0x20: PLLA_Reset


# und abschliessend den CLK0-Ausgang aktivieren
#----------------------------------------------
[3, 0x06]								
#Register 3, Register-Inhalt 0x06: "Output Disable for CLKx": Ausgang CLK0 freigeben, Ausgaenge CLK1/CLK2 bleiben gesperrt/inaktiv

]
 





###############################################################################
# Settings

# I2C-Kommunikation Initialisierung
# ---------------------------------

sdaPIN = machine.Pin(6) #I2C1 - SDA an uC Pin 6 - Achtung: beim SEEED XIAO Board mit dem uC RP2040 ist nur I2C1 auf Pins gelegt
sclPIN = machine.Pin(7) #I2C1 - SCL an uC Pin 7 - dies ist abweichend zum Raspberry Pi Pico-Board

i2c = machine.I2C(1, sda=sdaPIN, scl=sclPIN, freq=100000, timeout=50000)    #I2C mit Standard-Taktfrequenz 100kHz betrieben


###############################################################################
# Funktion, die die Daten in der obigen Liste in die Register des Si5351A schreibt

def reg_write(i2c, addr, reg, data):
    
    # Construct message
    msg = bytearray()
    msg.append(data)
    
    # Write out message to register
    i2c.writeto_mem(addr, reg, msg)
    
 


###############################################################################
# Main

for i in regs:          #jede Register-Adresse samt Inhalt aus obiger Liste wird an den Si5351A gesendet
    print (i[0])
    reg = i[0]
    print (i[1])
    data = i[1]

    reg_write(i2c, Si5351A_ADDR, reg, data)
    
    time.sleep_us(200)  #nach dem Laden jedes Registers 200usec warten, damit die I2C-Uebertragung sicher abgeschlossen ist


#---> NUN SOLLTE DAS SIGNAL MIT DER GEWUENSCHTEN FREQUENZ AM AUSGANG CLK0 ANSTEHEN







